@@ -26,7 +26,7 @@ int notcurses_inputready_fd(struct notcurses* n);
 typedef struct cell {
   uint32_t gcluster;          // 4B → 4B
   uint8_t gcluster_backstop;  // 1B → 5B (8 bits of zero)
-  uint8_t width;              // 1B → 6B (8 bits, width biased-1)
+  uint8_t width;              // 1B → 6B (8 bits, width)
   uint16_t stylemask;         // 2B → 8B (16 bits of NCSTYLE_* attributes)
   uint64_t channels;          // + 8b == 16b
 } cell;
@@ -411,7 +411,7 @@ void ncreader_destroy(struct ncreader* n, char** contents);
 int ncplane_puttext(struct ncplane* n, int y, ncalign_e align, const char* text, size_t* bytes);
 int ncplane_putnstr_yx(struct ncplane* n, int y, int x, size_t s, const char* gclusters);
 int ncplane_putnstr_aligned(struct ncplane* n, int y, ncalign_e align, size_t s, const char* gclustarr);
-int ncplane_qrcode(struct ncplane* n, ncblitter_e blitter, int* ymax, int* xmax, const void* data, size_t len);
+int ncplane_qrcode(struct ncplane* n, int* ymax, int* xmax, const void* data, size_t len);
 struct ncdirect* ncdirect_init(const char* termtype, FILE* fp, uint64_t flags);
 int ncdirect_set_bg_rgb8(struct ncdirect* n, unsigned r, unsigned g, unsigned b);
 int ncdirect_set_fg_rgb8(struct ncdirect* n, unsigned r, unsigned g, unsigned b);
