@@ -36,16 +36,17 @@ void notcurses_version_components(int* major, int* minor, int* patch, int* tweak
 
 // reset the current colors, styles, and palette. called on startup (to purge
 // any preexisting styling) and shutdown (to not affect further programs).
+// nc->ttyfd must be valid.
 static int
 reset_term_attributes(notcurses* nc){
   int ret = 0;
-  if(nc->tcache.op && term_emit(nc->tcache.op, nc->ttyfp, false)){
+  if(nc->tcache.op && tty_emit(nc->tcache.op, nc->ttyfd)){
     ret = -1;
   }
-  if(nc->tcache.sgr0 && term_emit(nc->tcache.sgr0, nc->ttyfp, false)){
+  if(nc->tcache.sgr0 && tty_emit(nc->tcache.sgr0, nc->ttyfd)){
     ret = -1;
   }
-  if(nc->tcache.oc && term_emit(nc->tcache.oc, nc->ttyfp, true)){
+  if(nc->tcache.oc && tty_emit(nc->tcache.oc, nc->ttyfd)){
     ret = -1;
   }
   ret |= notcurses_mouse_disable(nc);
@@ -61,8 +62,8 @@ notcurses_stop_minimal(void* vnc){
   int ret = drop_signals(nc);
   // be sure to write the restoration sequences *prior* to running rmcup, as
   // they apply to the screen (alternate or otherwise) we're actually using.
-  ret |= reset_term_attributes(nc);
   if(nc->ttyfd >= 0){
+    ret |= reset_term_attributes(nc);
     if(nc->tcache.rmcup && tty_emit(nc->tcache.rmcup, nc->ttyfd)){
       ret = -1;
     }
@@ -221,10 +222,12 @@ void free_plane(ncplane* p){
   if(p){
     // ncdirect fakes an ncplane with no ->pile
     if(ncplane_pile(p)){
+      notcurses* nc = ncplane_notcurses(p);
+      pthread_mutex_lock(&nc->statlock);
       --ncplane_notcurses(p)->stats.planes;
       ncplane_notcurses(p)->stats.fbbytes -= sizeof(*p->fb) * p->leny * p->lenx;
+      pthread_mutex_unlock(&nc->statlock);
       if(p->above == NULL && p->below == NULL){
-        struct notcurses* nc = ncplane_notcurses(p);
         pthread_mutex_lock(&nc->pilelock);
         ncpile_destroy(ncplane_pile(p));
         pthread_mutex_unlock(&nc->pilelock);
@@ -290,6 +293,9 @@ ncplane* ncplane_new_internal(notcurses* nc, ncplane* n,
     return NULL;
   }
   ncplane* p = malloc(sizeof(*p));
+  if(p == NULL){
+    return NULL;
+  }
   size_t fbsize = sizeof(*p->fb) * (nopts->rows * nopts->cols);
   if((p->fb = malloc(fbsize)) == NULL){
     logerror(nc, "Error allocating cellmatrix (r=%d, c=%d)\n",
@@ -354,8 +360,10 @@ ncplane* ncplane_new_internal(notcurses* nc, ncplane* n,
     }else{ // new pile
       make_ncpile(nc, p);
     }
+    pthread_mutex_lock(&nc->statlock);
     nc->stats.fbbytes += fbsize;
     ++nc->stats.planes;
+    pthread_mutex_unlock(&nc->statlock);
     pthread_mutex_unlock(&nc->pilelock);
   }
   loginfo(nc, "Created new %dx%d plane \"%s\" @ %dx%d\n",
@@ -538,6 +546,7 @@ int ncplane_resize_internal(ncplane* n, int keepy, int keepx, int keepleny,
     return -1;
   }
   loginfo(ncplane_notcurses(n), "%dx%d @ %d/%d → %d/%d @ %d/%d (keeping %dx%d from %d/%d)\n", rows, cols, n->absy, n->absx, ylen, xlen, n->absy + keepy + yoff, n->absx + keepx + xoff, keepleny, keeplenx, keepy, keepx);
+  notcurses* nc = ncplane_notcurses(n);
   // we're good to resize. we'll need alloc up a new framebuffer, and copy in
   // those elements we're retaining, zeroing out the rest. alternatively, if
   // we've shrunk, we will be filling the new structure.
@@ -556,8 +565,10 @@ int ncplane_resize_internal(ncplane* n, int keepy, int keepx, int keepleny,
     n->x = xlen - 1;
   }
   nccell* preserved = n->fb;
+  pthread_mutex_lock(&nc->statlock);
   ncplane_notcurses(n)->stats.fbbytes -= sizeof(*preserved) * (rows * cols);
   ncplane_notcurses(n)->stats.fbbytes += fbsize;
+  pthread_mutex_unlock(&nc->statlock);
   n->fb = fb;
   const int oldabsy = n->absy;
   // go ahead and move. we can no longer fail at this point. but don't yet
@@ -699,64 +710,20 @@ make_nonblocking(int fd){
 static void
 reset_stats(ncstats* stats){
   uint64_t fbbytes = stats->fbbytes;
+  unsigned planes = stats->planes;
   memset(stats, 0, sizeof(*stats));
   stats->render_min_ns = 1ull << 62u;
   stats->render_min_bytes = 1ull << 62u;
   stats->raster_min_ns = 1ull << 62u;
   stats->writeout_min_ns = 1ull << 62u;
   stats->fbbytes = fbbytes;
+  stats->planes = planes;
 }
 
-// add the current stats to the cumulative stashed stats, and reset them
-static void
-stash_stats(notcurses* nc){
-  nc->stashstats.renders += nc->stats.renders;
-  nc->stashstats.render_ns += nc->stats.render_ns;
-  nc->stashstats.raster_ns += nc->stats.raster_ns;
-  nc->stashstats.writeouts += nc->stats.writeouts;
-  nc->stashstats.writeout_ns += nc->stats.writeout_ns;
-  nc->stashstats.failed_renders += nc->stats.failed_renders;
-  nc->stashstats.failed_writeouts += nc->stats.failed_writeouts;
-  nc->stashstats.render_bytes += nc->stats.render_bytes;
-  if(nc->stashstats.render_max_bytes < nc->stats.render_max_bytes){
-    nc->stashstats.render_max_bytes = nc->stats.render_max_bytes;
-  }
-  if(nc->stashstats.render_max_ns < nc->stats.render_max_ns){
-    nc->stashstats.render_max_ns = nc->stats.render_max_ns;
-  }
-  if(nc->stashstats.render_min_bytes > nc->stats.render_min_bytes){
-    nc->stashstats.render_min_bytes = nc->stats.render_min_bytes;
-  }
-  if(nc->stashstats.render_min_ns > nc->stats.render_min_ns){
-    nc->stashstats.render_min_ns = nc->stats.render_min_ns;
-  }
-  if(nc->stashstats.writeout_max_ns < nc->stats.writeout_max_ns){
-    nc->stashstats.writeout_max_ns = nc->stats.writeout_max_ns;
-  }
-  if(nc->stashstats.writeout_min_ns > nc->stats.writeout_min_ns){
-    nc->stashstats.writeout_min_ns = nc->stats.writeout_min_ns;
-  }
-  if(nc->stashstats.raster_max_ns < nc->stats.raster_max_ns){
-    nc->stashstats.raster_max_ns = nc->stats.raster_max_ns;
-  }
-  if(nc->stashstats.raster_min_ns > nc->stats.raster_min_ns){
-    nc->stashstats.raster_min_ns = nc->stats.raster_min_ns;
-  }
-  nc->stashstats.cellelisions += nc->stats.cellelisions;
-  nc->stashstats.cellemissions += nc->stats.cellemissions;
-  nc->stashstats.fgelisions += nc->stats.fgelisions;
-  nc->stashstats.fgemissions += nc->stats.fgemissions;
-  nc->stashstats.bgelisions += nc->stats.bgelisions;
-  nc->stashstats.bgemissions += nc->stats.bgemissions;
-  nc->stashstats.defaultelisions += nc->stats.defaultelisions;
-  nc->stashstats.defaultemissions += nc->stats.defaultemissions;
-  nc->stashstats.refreshes += nc->stats.refreshes;
-  // fbbytes aren't stashed
-  reset_stats(&nc->stats);
-}
-
-void notcurses_stats(const notcurses* nc, ncstats* stats){
+void notcurses_stats(notcurses* nc, ncstats* stats){
+  pthread_mutex_lock(&nc->statlock);
   memcpy(stats, &nc->stats, sizeof(*stats));
+  pthread_mutex_unlock(&nc->statlock);
 }
 
 ncstats* notcurses_stats_alloc(const notcurses* nc __attribute__ ((unused))){
@@ -764,10 +731,12 @@ ncstats* notcurses_stats_alloc(const notcurses* nc __attribute__ ((unused))){
 }
 
 void notcurses_stats_reset(notcurses* nc, ncstats* stats){
+  pthread_mutex_lock(&nc->statlock);
   if(stats){
     memcpy(stats, &nc->stats, sizeof(*stats));
   }
-  stash_stats(nc);
+  reset_stats(&nc->stats);
+  pthread_mutex_unlock(&nc->statlock);
 }
 
 // unless the suppress_banner flag was set, print some version information and
@@ -810,6 +779,9 @@ init_banner(const notcurses* nc){
     if(!notcurses_canutf8(nc)){
       fprintf(stderr, "\n Warning! Encoding is not UTF-8; output may be degraded.\n");
     }
+    if(!nc->tcache.hpa){
+      fprintf(stderr, "\n Warning! No absolute horizontal placement.\n");
+    }
     if(nc->tcache.sgr0){
       term_emit(nc->tcache.sgr0, stderr, true);
     }
@@ -922,6 +894,7 @@ recursive_lock_init(pthread_mutex_t *lock){
 #endif
 }
 
+// FIXME cut this up into a few distinct pieces, yearrrgh
 notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
   notcurses_options defaultopts;
   memset(&defaultopts, 0, sizeof(defaultopts));
@@ -958,20 +931,13 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
   if(outfp == NULL){
     outfp = stdout;
   }
-  if(recursive_lock_init(&ret->pilelock)){
-    fprintf(stderr, "Couldn't initialize pile mutex\n");
-    free(ret);
-    return NULL;
-  }
   ret->margin_t = opts->margin_t;
   ret->margin_b = opts->margin_b;
   ret->margin_l = opts->margin_l;
   ret->margin_r = opts->margin_r;
   ret->cursory = ret->cursorx = -1;
-  ret->stats.fbbytes = 0;
-  ret->stashstats.fbbytes = 0;
+  memset(&ret->stats, 0, sizeof(ret->stats));
   reset_stats(&ret->stats);
-  reset_stats(&ret->stashstats);
   ret->ttyfp = outfp;
   ret->renderfp = opts->renderfp;
   memset(&ret->rstate, 0, sizeof(ret->rstate));
@@ -980,7 +946,6 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
   ret->lastframe = NULL;
   ret->lfdimy = 0;
   ret->lfdimx = 0;
-  ret->libsixel = false;
   egcpool_init(&ret->pool);
   if((ret->loglevel = opts->loglevel) > NCLOGLEVEL_TRACE || ret->loglevel < 0){
     fprintf(stderr, "Invalid loglevel %d\n", ret->loglevel);
@@ -1002,6 +967,16 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
   }else{
     fprintf(stderr, "Defaulting to %dx%d (output is not to a terminal)\n", DEFAULT_ROWS, DEFAULT_COLS);
   }
+  if(recursive_lock_init(&ret->pilelock)){
+    fprintf(stderr, "Couldn't initialize pile mutex\n");
+    free(ret);
+    return NULL;
+  }
+  if(pthread_mutex_init(&ret->statlock, NULL)){
+    pthread_mutex_destroy(&ret->pilelock);
+    free(ret);
+    return NULL;
+  }
   if(setup_signals(ret, (opts->flags & NCOPTION_NO_QUIT_SIGHANDLERS),
                    (opts->flags & NCOPTION_NO_WINCH_SIGHANDLER),
                    notcurses_stop_minimal)){
@@ -1025,12 +1000,9 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
             shortname_term ? shortname_term : "?",
             longname_term ? longname_term : "?");
   }
-  if(interrogate_terminfo(&ret->tcache, shortname_term)){
+  if(interrogate_terminfo(&ret->tcache, shortname_term, utf8)){
     goto err;
   }
-  ret->tcache.utf8 = utf8;
-  warn_terminfo(ret, &ret->tcache);
-  reset_term_attributes(ret);
   if(ncinputlayer_init(&ret->input, stdin)){
     goto err;
   }
@@ -1058,6 +1030,7 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
       free_plane(ret->stdplane);
       goto err;
     }
+    reset_term_attributes(ret);
   }
   if((ret->rstate.mstreamfp = open_memstream(&ret->rstate.mstream, &ret->rstate.mstrsize)) == NULL){
     free_plane(ret->stdplane);
@@ -1092,6 +1065,8 @@ err:
   // FIXME looks like we have some memory leaks on this error path?
   tcsetattr(ret->ttyfd, TCSANOW, &ret->tpreserved);
   drop_signals(ret);
+  pthread_mutex_destroy(&ret->statlock);
+  pthread_mutex_destroy(&ret->pilelock);
   free(ret);
   return NULL;
 }
@@ -1146,7 +1121,7 @@ int notcurses_stop(notcurses* nc){
     // wrote. move it to the bottom left of the screen.
     if(!nc->tcache.smcup){
       tty_emit(tiparm(nc->tcache.hpa, 0), nc->ttyfd);
-      tty_emit(tiparm(nc->tcache.vpa, nc->lfdimy - 1), nc->ttyfd);
+      tty_emit(tiparm(nc->tcache.vpa, nc->lfdimy + nc->margin_t - 1), nc->ttyfd);
     }
     if(nc->ttyfd >= 0){
       ret |= close(nc->ttyfd);
@@ -1155,70 +1130,70 @@ int notcurses_stop(notcurses* nc){
     free(nc->lastframe);
     free(nc->rstate.mstream);
     input_free_esctrie(&nc->input.inputescapes);
-    stash_stats(nc);
     if(!nc->suppress_banner){
-      if(nc->stashstats.renders){
+      if(nc->stats.renders){
         char totalbuf[BPREFIXSTRLEN + 1];
         char minbuf[BPREFIXSTRLEN + 1];
         char maxbuf[BPREFIXSTRLEN + 1];
         char avgbuf[BPREFIXSTRLEN + 1];
-        qprefix(nc->stashstats.render_ns, NANOSECS_IN_SEC, totalbuf, 0);
-        qprefix(nc->stashstats.render_min_ns, NANOSECS_IN_SEC, minbuf, 0);
-        qprefix(nc->stashstats.render_max_ns, NANOSECS_IN_SEC, maxbuf, 0);
-        qprefix(nc->stashstats.render_ns / nc->stashstats.renders, NANOSECS_IN_SEC, avgbuf, 0);
-        fprintf(stderr, "\n%ju render%s, %ss total (%ss min, %ss max, %ss avg)\n",
-                nc->stashstats.renders, nc->stashstats.renders == 1 ? "" : "s",
-                totalbuf, minbuf, maxbuf, avgbuf);
-        qprefix(nc->stashstats.raster_ns, NANOSECS_IN_SEC, totalbuf, 0);
-        qprefix(nc->stashstats.raster_min_ns, NANOSECS_IN_SEC, minbuf, 0);
-        qprefix(nc->stashstats.raster_max_ns, NANOSECS_IN_SEC, maxbuf, 0);
-        qprefix(nc->stashstats.raster_ns / nc->stashstats.writeouts, NANOSECS_IN_SEC, avgbuf, 0);
-        fprintf(stderr, "%ju raster%s, %ss total (%ss min, %ss max, %ss avg)\n",
-                nc->stashstats.writeouts, nc->stashstats.writeouts == 1 ? "" : "s",
-                totalbuf, minbuf, maxbuf, avgbuf);
-        qprefix(nc->stashstats.writeout_ns, NANOSECS_IN_SEC, totalbuf, 0);
-        qprefix(nc->stashstats.writeout_min_ns, NANOSECS_IN_SEC, minbuf, 0);
-        qprefix(nc->stashstats.writeout_max_ns, NANOSECS_IN_SEC, maxbuf, 0);
-        qprefix(nc->stashstats.writeouts ? nc->stashstats.writeout_ns / nc->stashstats.writeouts : 0,
+        qprefix(nc->stats.render_ns, NANOSECS_IN_SEC, totalbuf, 0);
+        qprefix(nc->stats.render_min_ns, NANOSECS_IN_SEC, minbuf, 0);
+        qprefix(nc->stats.render_max_ns, NANOSECS_IN_SEC, maxbuf, 0);
+        qprefix(nc->stats.render_ns / nc->stats.renders, NANOSECS_IN_SEC, avgbuf, 0);
+        fprintf(stderr, "\n%ju render%s, %ss (%ss min, %ss avg, %ss max)\n",
+                nc->stats.renders, nc->stats.renders == 1 ? "" : "s",
+                totalbuf, minbuf, avgbuf, maxbuf);
+        qprefix(nc->stats.raster_ns, NANOSECS_IN_SEC, totalbuf, 0);
+        qprefix(nc->stats.raster_min_ns, NANOSECS_IN_SEC, minbuf, 0);
+        qprefix(nc->stats.raster_max_ns, NANOSECS_IN_SEC, maxbuf, 0);
+        qprefix(nc->stats.raster_ns / nc->stats.writeouts, NANOSECS_IN_SEC, avgbuf, 0);
+        fprintf(stderr, "%ju raster%s, %ss (%ss min, %ss avg, %ss max)\n",
+                nc->stats.writeouts, nc->stats.writeouts == 1 ? "" : "s",
+                totalbuf, minbuf, avgbuf, maxbuf);
+        qprefix(nc->stats.writeout_ns, NANOSECS_IN_SEC, totalbuf, 0);
+        qprefix(nc->stats.writeout_min_ns, NANOSECS_IN_SEC, minbuf, 0);
+        qprefix(nc->stats.writeout_max_ns, NANOSECS_IN_SEC, maxbuf, 0);
+        qprefix(nc->stats.writeouts ? nc->stats.writeout_ns / nc->stats.writeouts : 0,
                 NANOSECS_IN_SEC, avgbuf, 0);
-        fprintf(stderr, "%ju write%s, %ss total (%ss min, %ss max, %ss avg)\n",
-                nc->stashstats.writeouts, nc->stashstats.writeouts == 1 ? "" : "s",
-                totalbuf, minbuf, maxbuf, avgbuf);
-        bprefix(nc->stashstats.render_bytes, 1, totalbuf, 0),
-        bprefix(nc->stashstats.render_min_bytes, 1, minbuf, 0),
-        bprefix(nc->stashstats.render_max_bytes, 1, maxbuf, 0),
-        fprintf(stderr, "%sB total (%sB min, %sB max, %.03gKiB avg)\n",
-                totalbuf, minbuf, maxbuf,
-                nc->stashstats.render_bytes / (double)nc->stashstats.renders / 1024);
+        fprintf(stderr, "%ju write%s, %ss (%ss min, %ss avg, %ss max)\n",
+                nc->stats.writeouts, nc->stats.writeouts == 1 ? "" : "s",
+                totalbuf, minbuf, avgbuf, maxbuf);
+        bprefix(nc->stats.render_bytes, 1, totalbuf, 1),
+        bprefix(nc->stats.render_min_bytes, 1, minbuf, 1),
+        bprefix(nc->stats.renders ? nc->stats.render_bytes / nc->stats.renders : 0, 1, avgbuf, 1);
+        bprefix(nc->stats.render_max_bytes, 1, maxbuf, 1),
+        fprintf(stderr, "%sB (%sB min, %sB avg, %sB max)\n",
+                totalbuf, minbuf, avgbuf, maxbuf);
       }
-      if(nc->stashstats.renders || nc->stashstats.failed_renders){
+      if(nc->stats.renders || nc->stats.failed_renders){
         fprintf(stderr, "%ju failed render%s, %ju failed write%s, %ju refresh%s\n",
-                nc->stashstats.failed_renders,
-                nc->stashstats.failed_renders == 1 ? "" : "s",
-                nc->stashstats.failed_writeouts,
-                nc->stashstats.failed_writeouts == 1 ? "" : "s",
-                nc->stashstats.refreshes,
-                nc->stashstats.refreshes == 1 ? "" : "es");
+                nc->stats.failed_renders,
+                nc->stats.failed_renders == 1 ? "" : "s",
+                nc->stats.failed_writeouts,
+                nc->stats.failed_writeouts == 1 ? "" : "s",
+                nc->stats.refreshes,
+                nc->stats.refreshes == 1 ? "" : "es");
         fprintf(stderr, "RGB emits:elides: def %ju:%ju fg %ju:%ju bg %ju:%ju\n",
-                nc->stashstats.defaultemissions,
-                nc->stashstats.defaultelisions,
-                nc->stashstats.fgemissions,
-                nc->stashstats.fgelisions,
-                nc->stashstats.bgemissions,
-                nc->stashstats.bgelisions);
+                nc->stats.defaultemissions,
+                nc->stats.defaultelisions,
+                nc->stats.fgemissions,
+                nc->stats.fgelisions,
+                nc->stats.bgemissions,
+                nc->stats.bgelisions);
         fprintf(stderr, "Cell emits:elides: %ju/%ju (%.2f%%) %.2f%% %.2f%% %.2f%%\n",
-                nc->stashstats.cellemissions, nc->stashstats.cellelisions,
-                (nc->stashstats.cellemissions + nc->stashstats.cellelisions) == 0 ? 0 :
-                (nc->stashstats.cellelisions * 100.0) / (nc->stashstats.cellemissions + nc->stashstats.cellelisions),
-                (nc->stashstats.defaultemissions + nc->stashstats.defaultelisions) == 0 ? 0 :
-                (nc->stashstats.defaultelisions * 100.0) / (nc->stashstats.defaultemissions + nc->stashstats.defaultelisions),
-                (nc->stashstats.fgemissions + nc->stashstats.fgelisions) == 0 ? 0 :
-                (nc->stashstats.fgelisions * 100.0) / (nc->stashstats.fgemissions + nc->stashstats.fgelisions),
-                (nc->stashstats.bgemissions + nc->stashstats.bgelisions) == 0 ? 0 :
-                (nc->stashstats.bgelisions * 100.0) / (nc->stashstats.bgemissions + nc->stashstats.bgelisions));
+                nc->stats.cellemissions, nc->stats.cellelisions,
+                (nc->stats.cellemissions + nc->stats.cellelisions) == 0 ? 0 :
+                (nc->stats.cellelisions * 100.0) / (nc->stats.cellemissions + nc->stats.cellelisions),
+                (nc->stats.defaultemissions + nc->stats.defaultelisions) == 0 ? 0 :
+                (nc->stats.defaultelisions * 100.0) / (nc->stats.defaultemissions + nc->stats.defaultelisions),
+                (nc->stats.fgemissions + nc->stats.fgelisions) == 0 ? 0 :
+                (nc->stats.fgelisions * 100.0) / (nc->stats.fgemissions + nc->stats.fgelisions),
+                (nc->stats.bgemissions + nc->stats.bgelisions) == 0 ? 0 :
+                (nc->stats.bgelisions * 100.0) / (nc->stats.bgemissions + nc->stats.bgelisions));
       }
     }
     del_curterm(cur_term);
+    ret |= pthread_mutex_destroy(&nc->statlock);
     ret |= pthread_mutex_destroy(&nc->pilelock);
     free(nc);
   }
@@ -2078,7 +2053,7 @@ bool notcurses_canchangecolor(const notcurses* nc){
 }
 
 bool notcurses_cansixel(const notcurses* nc){
-  return nc->libsixel;
+  return nc->tcache.libsixel;
 }
 
 palette256* palette256_new(notcurses* nc){
