@@ -79,8 +79,7 @@ auto ncvisual_geom(const notcurses* nc, const ncvisual* n,
     blitter = vopts->blitter;
   }
   const bool maydegrade = !(vopts && (vopts->flags & NCVISUAL_OPTION_NODEGRADE));
-  const struct blitset* bset = lookup_blitset(notcurses_canutf8(nc),
-                                              blitter, maydegrade);
+  const struct blitset* bset = lookup_blitset(&nc->tcache, blitter, maydegrade);
   if(!bset){
     return -1;
   }
