@@ -8,6 +8,7 @@
 #include <libgen.h>
 #include <unistd.h>
 #include <iostream>
+#include <ncpp/Direct.hh>
 #include <ncpp/Visual.hh>
 #include <ncpp/NotCurses.hh>
 #include "compat/compat.h"
@@ -22,7 +23,7 @@ void usage(std::ostream& o, const char* name, int exitcode){
   o << " -h: display help and exit with success\n";
   o << " -V: print program name and version\n";
   o << " -q: be quiet (no frame/timing information along top of screen)\n";
-  o << " -k: don't use the alternate screen\n";
+  o << " -k: use direct mode (cannot be used with -L or -d)\n";
   o << " -L: loop frames\n";
   o << " -t seconds: delay t seconds after each file\n";
   o << " -l loglevel: integer between 0 and 9, goes to stderr'\n";
@@ -195,10 +196,18 @@ auto handle_opts(int argc, char** argv, notcurses_options& opts, bool* quiet,
           usage(std::cerr, argv[0], EXIT_FAILURE);
         }
         break;
-      case 'k':{
+      case 'k':{ // actually engages direct mode
         opts.flags |= NCOPTION_NO_ALTERNATE_SCREEN;
+        if(*loop || *timescale != 1.0){
+          std::cerr << "-k cannot be used with -L or -d" << std::endl;
+          usage(std::cerr, argv[0], EXIT_FAILURE);
+        }
         break;
       }case 'L':{
+        if(opts.flags & NCOPTION_NO_ALTERNATE_SCREEN){
+          std::cerr << "-L cannot be used with -k" << std::endl;
+          usage(std::cerr, argv[0], EXIT_FAILURE);
+        }
         *loop = true;
         break;
       }case 'm':{
@@ -231,6 +240,10 @@ auto handle_opts(int argc, char** argv, notcurses_options& opts, bool* quiet,
           usage(std::cerr, argv[0], EXIT_FAILURE);
         }
         *timescale = ts;
+        if(opts.flags & NCOPTION_NO_ALTERNATE_SCREEN){
+          std::cerr << "-d cannot be used with -k" << std::endl;
+          usage(std::cerr, argv[0], EXIT_FAILURE);
+        }
         break;
       }case 'l':{
         std::stringstream ss;
@@ -259,6 +272,29 @@ auto handle_opts(int argc, char** argv, notcurses_options& opts, bool* quiet,
   return optind;
 }
 
+// argc/argv ought already be reduced to only the media arguments
+int direct_mode_player(int argc, char** argv, ncscale_e scalemode, ncblitter_e blitter){
+  Direct dm{};
+  if(!dm.canopen_images()){
+    std::cerr << "Notcurses was compiled without multimedia support\n";
+    return -1;
+  }
+  bool failed = false;
+  {
+    for(auto i = 0 ; i < argc ; ++i){
+      try{
+        dm.render_image(argv[i], NCALIGN_RIGHT, blitter, scalemode);
+      }catch(std::exception& e){
+        // FIXME want to stop nc first :/ can't due to stdn, ugh
+        std::cerr << argv[i] << ": " << e.what() << "\n";
+        failed = true;
+        break;
+      }
+    }
+  }
+  return failed ? -1 : 0;
+}
+
 auto main(int argc, char** argv) -> int {
   if(setlocale(LC_ALL, "") == nullptr){
     std::cerr << "Couldn't set locale based off LANG\n";
@@ -272,6 +308,15 @@ auto main(int argc, char** argv) -> int {
   bool loop = false;
   auto nonopt = handle_opts(argc, argv, ncopts, &quiet, &timescale, &scalemode,
                             &blitter, &displaytime, &loop);
+  // if -k was provided, we now use direct mode rather than simply not using the
+  // alternate screen, so that output is inline with the shell.
+  if(ncopts.flags & NCOPTION_NO_ALTERNATE_SCREEN){
+    if(direct_mode_player(argc - nonopt, argv + nonopt, scalemode, blitter)){
+      return EXIT_FAILURE;
+    }
+    return EXIT_SUCCESS;
+  }
+  // no -k, we're using full rendered mode (and the alternate screen).
   ncopts.flags |= NCOPTION_INHIBIT_SETLOCALE;
   if(quiet){
     ncopts.flags |= NCOPTION_SUPPRESS_BANNERS;
@@ -296,9 +341,7 @@ auto main(int argc, char** argv) -> int {
         failed = true;
         break;
       }
-      if(!(ncopts.flags & NCOPTION_NO_ALTERNATE_SCREEN)){
-        stdn->erase();
-      }
+      stdn->erase();
       struct ncvisual_options vopts{};
       int r;
       vopts.n = *stdn;
